// (c) Peter Kankowski, 2015. http://www.strchr.com kankowski@gmail.com
#include <assert.h>
#include <stdio.h>

typedef enum {
	CALENDAR_JULIAN = 0,
	CALENDAR_GREGORIAN = 1
} CALENDAR_TYPE;

unsigned dayofweek(unsigned year, unsigned month, unsigned day, CALENDAR_TYPE type) {
	assert(5 <= year && year <= 9999);
	assert(1 <= month && month <= 12);
	
	static const unsigned char month_lengths[] = {0, 31, 29, 31, 30, 31, 30,
		31, 31, 30, 31, 30, 31};
	
	static const unsigned char month_shifts[] = {0, 5, 1, 0, 3, 5, 1, 3, 6, 2, 4, 0, 2};
	// a = [31, 30, 31, 30, 31, 31, 30, 31, 30, 31, 31, 28]
	// for i in range(len(a)): print(sum(a[0:i+1])%7, end = ', ')
	
	assert(1 <= day && day <= month_lengths[month]);

	year -= month <= 2;
	
	unsigned dow = year + year / 4 + month_shifts[month] + day;
	
	unsigned century = year / 100;

	dow -= (century - century / 4 - 2) & (type != CALENDAR_JULIAN ? -1 : 0);
	
	return dow % 7;
}

#define MAKE_DATE(month, day) (((unsigned)(day) << 8) | (unsigned)(month))
#define GET_DAY(date) ((unsigned)(date) >> 8)
#define GET_MONTH(date) ((unsigned)(date) & 0xFF)

unsigned roshhashanah(unsigned year, CALENDAR_TYPE type) {
	assert(sizeof(unsigned) >= 4); // 32-bit integer type is required
	assert(5 <= year && year <= 9999);
	
	unsigned r = (12 * year + 12) % 19;

	// Calculate the number of parts (Talmudic units)
	int parts = 765433 * r - 1565 * year - 445405 + 123120 * (year % 4);
	
	// Take into account the difference between Gregorian and Julian calendars
	unsigned century = year / 100;
	unsigned gregorian_shift = (century - century / 4 - 2) & (type != CALENDAR_JULIAN ? -1 : 0);
	
	parts -= parts <= 0 ? 492479 : 0; // Floored division
	
	int day = parts / 492480 + gregorian_shift;
	
	parts %= 492480;
	
	// Postponement rules
	unsigned dow = (year + year / 4 - gregorian_shift + day + 2) % 7;
	
	day += (dow == 0) | (dow == 3) | (dow == 5);

	day += (dow == 1) & (parts >= 442111) & (r > 11);
	
	day += 2 * ((dow == 2) & (parts >= 311676) & (r > 6));
	
	// Calculate month and day
	int is_october = (day > 30);
	int is_not_august = (day > 0);
	
	unsigned month = 8 + is_not_august + is_october;
	
	day += is_not_august ? 0 : 31;
	day -= is_october ? 30 : 0;
	
	return MAKE_DATE(month, day);
}

typedef enum {
	EASTER_WESTERN, // Catholic and protestant churches
	EASTER_ORTHODOX, // Orthodox church
	EASTER_JULIAN // Before the transition to Gregorian calendar
} EASTER_TYPE;

unsigned easter(unsigned year, EASTER_TYPE type) {
	assert(5 <= year && year <= 9999);
	// Gauss algorithm implementation

	// Calculate the difference between Julian and Gregorian calendars for the given year
	unsigned century = year / 100;
	unsigned gregorian_shift = century - century / 4 - 2;
	
	unsigned x = 15, y = 6;
	
	// Metonic cycle correction
	x += (2 - (13 + 8 * century) / 25 + gregorian_shift) & (type == EASTER_WESTERN ? -1 : 0);
	y += gregorian_shift & (type == EASTER_WESTERN ? -1 : 0);
	
	// The core formula
	unsigned g = year % 19;
	unsigned d = (g * 19 + x) % 30; // Paschal Full Moon
	unsigned e = (2 * (year % 4) + 4 * year - d + y) % 7; // Sunday after PFM
	
	unsigned day = d + e;

	// Correction for the length of the moon month
	day -= (e == 6) & ( (d == 29) | ((d == 28) & (g > 10)) ) ? 7 : 0;
	
	// Convert Orthodox Easter to Grigorian calendar
	day += gregorian_shift & (type == EASTER_ORTHODOX ? -1 : 0);
	
	// Calculate month and day
	int is_may = (day >= 40);
	int is_not_march = (day >= 10);

	unsigned month = 3 + is_not_march + is_may;
	day += 22 - (is_not_march ? 31 : 0) - (is_may ? 30 : 0);

	return MAKE_DATE( month, day );
}



int main() {
	assert( dayofweek(1905, 1, 9, CALENDAR_JULIAN) == 0 ); // "Bloody Sunday" (Revolution in Russia)
	assert( dayofweek(1410, 7, 15, CALENDAR_JULIAN) == 2 ); // Battle of Grunwald
	assert( dayofweek(1492, 10, 12, CALENDAR_JULIAN) == 5 ); // Columbus discovered the NewWorld
	assert( dayofweek(1452, 4, 15, CALENDAR_JULIAN) == 6 ); // Leonardo da Vinci birthday
	
	assert( dayofweek(1945, 5, 9, CALENDAR_GREGORIAN) == 3 ); // V-E day
	assert( dayofweek(1776, 7, 4, CALENDAR_GREGORIAN) == 4 ); // Independence Day in US
	assert( dayofweek(1582, 10, 15, CALENDAR_GREGORIAN) == 5 ); // Gregorian calendar introduced in Italy
	assert( dayofweek(1961, 4, 12, CALENDAR_GREGORIAN) == 3 ); // First human in space
	assert( dayofweek(1989, 11, 9, CALENDAR_GREGORIAN) == 4 ); // The fall of the Berlin Wall
	
	// Different years
	assert( dayofweek(2000, 1, 1, CALENDAR_GREGORIAN) == 6 );
	assert( dayofweek(2001, 1, 1, CALENDAR_GREGORIAN) == 1 );
	assert( dayofweek(2012, 1, 1, CALENDAR_GREGORIAN) == 0 );
	assert( dayofweek(2015, 1, 1, CALENDAR_GREGORIAN) == 4 );
	
	// Random dates in different months
	assert( dayofweek(2001, 9, 1, CALENDAR_GREGORIAN) == 6 );
	assert( dayofweek(2001, 2, 28, CALENDAR_GREGORIAN) == 3 );
	assert( dayofweek(2012, 3, 1, CALENDAR_GREGORIAN) == 4 );
	assert( dayofweek(2012, 2, 29, CALENDAR_GREGORIAN) == 3 );
	assert( dayofweek(2012, 4, 1, CALENDAR_GREGORIAN) == 0 );
	assert( dayofweek(2012, 5, 20, CALENDAR_GREGORIAN) == 0 );
	assert( dayofweek(2012, 6, 1, CALENDAR_GREGORIAN) == 5 );
	assert( dayofweek(2012, 7, 1, CALENDAR_GREGORIAN) == 0 );
	assert( dayofweek(2012, 8, 1, CALENDAR_GREGORIAN) == 3 );
	assert( dayofweek(2012, 9, 1, CALENDAR_GREGORIAN) == 6 );
	assert( dayofweek(2012, 10, 1, CALENDAR_GREGORIAN) == 1 );
	assert( dayofweek(2012, 11, 1, CALENDAR_GREGORIAN) == 4 );
	assert( dayofweek(2012, 11, 15, CALENDAR_GREGORIAN) == 4 );
	assert( dayofweek(2014, 12, 31, CALENDAR_GREGORIAN) == 3 );
	assert( dayofweek(2000, 9, 29, CALENDAR_GREGORIAN) == 5 );
	
	// Rosh Hashanah
	assert( roshhashanah(1998, CALENDAR_GREGORIAN) == MAKE_DATE(9, 21) );
	assert( roshhashanah(2000, CALENDAR_GREGORIAN) == MAKE_DATE(9, 30) );
	assert( roshhashanah(2005, CALENDAR_GREGORIAN) == MAKE_DATE(10, 4) );
	assert( roshhashanah(2014, CALENDAR_GREGORIAN) == MAKE_DATE(9, 25) );
	assert( roshhashanah(2015, CALENDAR_GREGORIAN) == MAKE_DATE(9, 14) );
	assert( roshhashanah(2016, CALENDAR_GREGORIAN) == MAKE_DATE(10, 3) );
	assert( roshhashanah(2017, CALENDAR_GREGORIAN) == MAKE_DATE(9, 21) );
	assert( roshhashanah(2018, CALENDAR_GREGORIAN) == MAKE_DATE(9, 10) );
	assert( roshhashanah(2028, CALENDAR_GREGORIAN) == MAKE_DATE(9, 21) );
	assert( roshhashanah(2043, CALENDAR_GREGORIAN) == MAKE_DATE(10, 5) );

	// Easter
	assert( easter(1700, EASTER_WESTERN) == MAKE_DATE(4, 11) );
	assert( easter(1701, EASTER_WESTERN) == MAKE_DATE(3, 27) );
	assert( easter(1702, EASTER_WESTERN) == MAKE_DATE(4, 16) );
	assert( easter(1850, EASTER_WESTERN) == MAKE_DATE(3, 31) );
	assert( easter(1910, EASTER_WESTERN) == MAKE_DATE(3, 27) );
	assert( easter(1987, EASTER_WESTERN) == MAKE_DATE(4, 19) );
	assert( easter(1995, EASTER_WESTERN) == MAKE_DATE(4, 16) );
	assert( easter(1999, EASTER_WESTERN) == MAKE_DATE(4, 4) );
	assert( easter(2000, EASTER_WESTERN) == MAKE_DATE(4, 23) );
	assert( easter(2013, EASTER_WESTERN) == MAKE_DATE(3, 31) );
	assert( easter(2014, EASTER_WESTERN) == MAKE_DATE(4, 20) );
	assert( easter(2015, EASTER_WESTERN) == MAKE_DATE(4, 5) );
	assert( easter(2016, EASTER_WESTERN) == MAKE_DATE(3, 27) );
	assert( easter(2017, EASTER_WESTERN) == MAKE_DATE(4, 16) );
	assert( easter(2018, EASTER_WESTERN) == MAKE_DATE(4, 1) );
	assert( easter(2019, EASTER_WESTERN) == MAKE_DATE(4, 21) );
	assert( easter(2020, EASTER_WESTERN) == MAKE_DATE(4, 12) );
	assert( easter(2298, EASTER_WESTERN) == MAKE_DATE(4, 3) );
	assert( easter(2299, EASTER_WESTERN) == MAKE_DATE(4, 16) );

	assert( easter(1920, EASTER_ORTHODOX) == MAKE_DATE(4, 11) );
	assert( easter(1987, EASTER_ORTHODOX) == MAKE_DATE(4, 19) );
	assert( easter(1995, EASTER_ORTHODOX) == MAKE_DATE(4, 23) );
	assert( easter(1999, EASTER_ORTHODOX) == MAKE_DATE(4, 11) );
	assert( easter(2000, EASTER_ORTHODOX) == MAKE_DATE(4, 30) );
	assert( easter(2013, EASTER_ORTHODOX) == MAKE_DATE(5, 5) );
	assert( easter(2014, EASTER_ORTHODOX) == MAKE_DATE(4, 20) );
	assert( easter(2015, EASTER_ORTHODOX) == MAKE_DATE(4, 12) );
	assert( easter(2016, EASTER_ORTHODOX) == MAKE_DATE(5, 1) );
	assert( easter(2017, EASTER_ORTHODOX) == MAKE_DATE(4, 16) );
	assert( easter(2018, EASTER_ORTHODOX) == MAKE_DATE(4, 8) );
	assert( easter(2019, EASTER_ORTHODOX) == MAKE_DATE(4, 28) );
	assert( easter(2020, EASTER_ORTHODOX) == MAKE_DATE(4, 19) );
	assert( easter(2298, EASTER_ORTHODOX) == MAKE_DATE(5, 8) );
	assert( easter(2299, EASTER_ORTHODOX) == MAKE_DATE(4, 23) );

	assert( easter(532, EASTER_JULIAN) == MAKE_DATE(4, 11) );
	assert( easter(533, EASTER_JULIAN) == MAKE_DATE(3, 27) );
	assert( easter(534, EASTER_JULIAN) == MAKE_DATE(4, 16) );
	assert( easter(535, EASTER_JULIAN) == MAKE_DATE(4, 8) );
	assert( easter(536, EASTER_JULIAN) == MAKE_DATE(3, 23) );
	assert( easter(537, EASTER_JULIAN) == MAKE_DATE(4, 12) );
	assert( easter(538, EASTER_JULIAN) == MAKE_DATE(4, 4) );
	assert( easter(539, EASTER_JULIAN) == MAKE_DATE(4, 24) );
	assert( easter(540, EASTER_JULIAN) == MAKE_DATE(4, 8) );
	assert( easter(541, EASTER_JULIAN) == MAKE_DATE(3, 31) );
	assert( easter(542, EASTER_JULIAN) == MAKE_DATE(4, 20) );
	assert( easter(543, EASTER_JULIAN) == MAKE_DATE(4, 5) );
	assert( easter(544, EASTER_JULIAN) == MAKE_DATE(3, 27) );
	assert( easter(545, EASTER_JULIAN) == MAKE_DATE(4, 16) );
	assert( easter(546, EASTER_JULIAN) == MAKE_DATE(4, 8) );
	assert( easter(547, EASTER_JULIAN) == MAKE_DATE(3, 24) );
	assert( easter(548, EASTER_JULIAN) == MAKE_DATE(4, 12) );
	assert( easter(549, EASTER_JULIAN) == MAKE_DATE(4, 4) );
	assert( easter(550, EASTER_JULIAN) == MAKE_DATE(4, 24) );

	for (unsigned year = 2010; year < 2020; year++) {
		unsigned easter_date = easter(year, EASTER_WESTERN);
		unsigned orthodox_easter_date = easter(year, EASTER_ORTHODOX);
		unsigned roshhashanah_date = roshhashanah(year, CALENDAR_GREGORIAN);
		printf("%u: %u %2u.%u %2u.%u %2u.%u\n", year, dayofweek(year, 1, 1, CALENDAR_GREGORIAN),
			GET_DAY(easter_date), GET_MONTH(easter_date),
			GET_DAY(orthodox_easter_date), GET_MONTH(orthodox_easter_date),
			GET_DAY(roshhashanah_date), GET_MONTH(roshhashanah_date));
	}
	
	printf("%d\n", dayofweek(0, 3, 1, CALENDAR_JULIAN));
	return 0;
}

